'
'  This program uses the system bios and low-level DOS calls to
' investigate various aspects (mainly attached equipment) of a computer
' system.
'
'  Declare necessary functions and define REGISTERS variable type for
' interrupt calls.
'
DECLARE FUNCTION BIN$(N AS LONG)
DECLARE FUNCTION DEC%(S AS STRING)
TYPE REGISTERS
AX AS INTEGER
BX AS INTEGER
CX AS INTEGER
DX AS INTEGER
BP AS INTEGER
SI AS INTEGER
DI AS INTEGER
FLAGS AS INTEGER
DS AS INTEGER
ES AS INTEGER
END TYPE
'
'  Define register varables and other data to be output based on system
' queries.
'
DIM INREGS AS REGISTERS,OUTREGS AS REGISTERS,BIT AS INTEGER,ARCH$(1),EBDA$(1)
DIM DISK$(1),MATH$(1),SYST$(3),VID$(1 TO 3),GAME$(1),POINT$(1),MODEM$(1)
DIM BIT0 AS INTEGER,BIT3 AS INTEGER,BYTE AS LONG,EBDAADD AS LONG,FREMRY AS LONG
DIM CMODE AS INTEGER,PSP AS LONG,COM1ADD AS INTEGER,COM2ADD AS INTEGER
DIM LPT1ADD AS INTEGER,LPT2ADD AS INTEGER,LPT3ADD AS INTEGER,NEWMODE AS INTEGER
DISK$(0)="No disk drive installed" : DISK$(1)="Disk drive installed"
MATH$(0)="No math coprocessor" : MATH$(1)="Math coprocessor installed"
SYST$(0)="16K" : SYST$(1)="32K" : SYST$(2)="48K" : SYST$(3)="64K"
VID$(1)="40 x 25 color" : VID$(2)="80 x 25 color" : VID$(3)="80 x 25 mono"
GAME$(0)="No game adapter detected" : GAME$(1)="Game adapter detected"
POINT$(0)="No pointing device detected" : POINT$(1)="Pointing device detected"
MODEM$(0)="No modem" : MODEM$(1)="Modem installed"
ARCH$(0)="PC bus I/O channel" : ARCH$(1)="Microchannel architecture detected"
EBDA$(0)="EBDA not allocated" : EBDA$(1)="EBDA allocated"
'
'  Get current video mode and then put screen into QB standard text mode.
'
INREGS.AX=&HF00
CALL INTERRUPTX(&H10,INREGS,OUTREGS)
CMODE=OUTREGS.AX-256*INT(OUTREGS.AX/256+.001)
SCREEN 0
CLS
'
'  Get new bios video mode for later test of whether or not it changed.
'
INREGS.AX=&HF00
CALL INTERRUPTX(&H10,INREGS,OUTREGS)
NEWMODE=OUTREGS.AX-256*INT(OUTREGS.AX/256+.001)
'
'  Don't do anything else unless DOS version is at least 2.
'
INREGS.AX=&H3000
CALL INTERRUPTX(&H21,INREGS,OUTREGS)
DOSVER=OUTREGS.AX-256*INT(OUTREGS.AX/256+.001)
IF DOSVER>1 THEN
'
'  Use interrupt 15 / function C0 to get information about computer model.
' (Some of the information returned from this and other interrupt calls
' needs to be extracted at the bit level.  Function BIN$ is used to
' convert numbers to binary for this purpose.  The MID$ function is then
' used to extract the bits.)
'
INREGS.AX=&HC000
CALL INTERRUPTX(&H15,INREGS,OUTREGS)
'
'  If AH or CF aren't 0, warn user about possible inapplicability of this
' program.
'
BYTE=CLNG(OUTREGS.FLAGS)
IF INT(OUTREGS.AX/256+.001)<>0 OR MID$(BIN$(BYTE),16,1)="1" THEN
PRINT "  This program may give some erroneous results for this computer.  (Int";
PRINT "errupt"
PRINT "15 / function C0 either didn't return a 0 in AH or else it set the carr";
PRINT "y flag."
PRINT "If the following output seems consistent with what you know about your ";
PRINT "computer,"
PRINT "then simply disregard this warning.)"
PRINT
PRINT "Press a key to continue..."
5 IF INKEY$="" THEN GOTO 5
CLS
END IF
DEF SEG=OUTREGS.ES
OS=OUTREGS.BX
MODEL=PEEK(OS+2)
SUBMOD=PEEK(OS+3)
BYTE=CLNG(PEEK(OS+5))
B$=BIN$(BYTE)
DEF SEG
IF MODEL=&HFA OR MODEL=&HF8 OR (MODEL=&HFC AND (SUBMOD=4 OR SUBMOD=5)) THEN
COMP$="PS/2"
ELSE
IF MODEL=&HFF THEN COMP$="PC"
IF MODEL=&HFE OR MODEL=&HFB THEN COMP$="PC XT"
IF MODEL=&HFD THEN COMP$="PCjr"
IF MODEL=&HFC THEN COMP$="AT (or higher)"
IF MODEL=&HFC AND SUBMOD=1 THEN COMP$="AT (or higher) or COMPAQ 286"
IF MODEL=&HFC AND SUBMOD=2 THEN COMP$="PC XT 286"
IF MODEL=&HF9 THEN COMP$="PC Convertible"
IF MODEL=&HFC AND SUBMOD>2 THEN COMP$="AT or higher"
END IF
PRINT COMP$;" computer model detected";
IF COMP$="PS/2" THEN
IF MODEL=&HFA THEN PRINT " (MODEL 30)"
IF MODEL=&HFC AND SUBMOD=4 THEN PRINT " (MODEL 50)"
IF MODEL=&HFC AND SUBMOD=5 THEN PRINT " (MODEL 60)"
IF MODEL=&HF8 THEN PRINT " (MODEL 80)"
ELSE
PRINT
END IF
PRINT
BIT=VAL(MID$(B$,15,1))
PRINT ARCH$(BIT)
PRINT
BIT3=VAL(MID$(B$,14,1))
PRINT EBDA$(BIT3)
PRINT
IF MID$(B$,11,1)="1" THEN PRINT "Real-time clock present" : PRINT
'
'  Interrupt 11 returns a "rudimentary list of equipment attached to the
' computer."
'
CALL INTERRUPTX(&H11,INREGS,OUTREGS)
'
'  Equipment presence is indicated bit by bit and the interpretation of
' the output (in AX) depends on the computer model.
'
BYTE=CLNG(OUTREGS.AX)
B$=BIN$(BYTE)
BIT0=VAL(MID$(B$,16,1))
PRINT DISK$(BIT0)
PRINT
BIT=VAL(MID$(B$,15,1))
PRINT MATH$(BIT)
PRINT
IF COMP$<>"PS/2" THEN
BIT=DEC(MID$(B$,13,2))
PRINT SYST$(BIT);" system ram"
PRINT
ELSE
BIT=VAL(MID$(B$,14,1))
PRINT POINT$(BIT)
PRINT
END IF
BIT=DEC(MID$(B$,11,2))
PRINT VID$(BIT);" video mode"
PRINT
IF BIT0=1 THEN
BIT=DEC(MID$(B$,9,2))
PRINT LTRIM$(RTRIM$(STR$(BIT)));" floppy drives installed"
PRINT
END IF
BIT=DEC(MID$(B$,5,3))
PRINT LTRIM$(RTRIM$(STR$(BIT)));" serial cards installed"
'
'  Pause when screen gets full.
'
PRINT
PRINT
PRINT "Press a key to continue...";
10 IF INKEY$="" THEN GOTO 10
PRINT
PRINT
PRINT
IF COMP$<>"PS/2" THEN
BIT=VAL(MID$(B$,4,1))
PRINT GAME$(BIT)
ELSE
BIT=VAL(MID$(B$,3,1))
PRINT MODEM$(BIT)
END IF
PRINT
BIT=DEC(MID$(B$,1,2))
PRINT LTRIM$(RTRIM$(STR$(BIT)));" printer ports installed"
PRINT
'
'  Interrupt 12 returns the amount of standard RAM installed.
'
CALL INTERRUPTX(&H12,INREGS,OUTREGS)
CONMEM=OUTREGS.AX
'
'  The interpretation of interrupt 12's output depends on whether the
' the computer is a PS/2 or not.
'
IF BIT3=1 THEN
PRINT LTRIM$(RTRIM$(STR$(CONMEM)));"K conventional memory (not counting EBDA)"
'
'  For PS/2 computers, interrupt 15 / function C1 returns the location of
' the extended bios data area.  This can be used to calculate the amount
' of extended bios memory (on the assumption that it ends at the byte
' before A000h).
'
INREGS.AX=&HC100
CALL INTERRUPTX(&H15,INREGS,OUTREGS)
EBDAADD=OUTREGS.ES
IF EBDAADD<0 THEN EBDAADD=EBDAADD+65536&
EBDAMEM=(&HA000&-EBDAADD)/64
PRINT LTRIM$(RTRIM$(STR$(EBDAMEM)));"K EBDA memory at ";HEX$(CINT(EBDAADD));
PRINT ":0000"
ELSE
PRINT LTRIM$(RTRIM$(STR$(CONMEM)));"K conventional memory"
END IF
PRINT
'
'  Interrupt 21 / function 62 returns the segment number specifying the
' bottom of available RAM.  It can be used with the output of interrupt
' 12 to calculate how much free memory is available.
'
INREGS.AX=&H6200
CALL INTERRUPTX(&H21,INREGS,OUTREGS)
PSP=CLNG(OUTREGS.BX) : IF PSP<0 THEN PSP=PSP+65536&
FREMRY=CONMEM*1024&-PSP*16&
PRINT "(";LTRIM$(RTRIM$(STR$(FREMRY)));" bytes free)"
PRINT
'
'  For relevant computer types, interrupt 15 / function 88 returns the
' amount of free extended memory.
'
IF MODEL=&HFC OR (COMP$="PS/2" AND MODEL<>&HFA) THEN
INREGS.AX=&H8800
CALL INTERRUPTX(&H15,INREGS,OUTREGS)
PRINT LTRIM$(RTRIM$(STR$(OUTREGS.AX)));"K extended memory (above 100,000h) ";
PRINT "available"
PRINT
END IF
'
'  Detecting the presence of COM ports is somewhat tricky (if only because
' I'm not totally sure of the best way of doing it).  The following is a
' roundabout way that seems to work.  COM1 and COM2 are detected by seeing
' if nonzero values for their addresses are stored at memory addresses
' 400h and 402h.  (LPT1, LPT2, and LPT3 are detected similarly.)  COM3 and
' and COM4 are detected by using interrupt 14 / function 30 to query their
' status.  If AH is 96 (decimal) or less, the COM port is assumed to
' exist.  (I'm not guarantying that this process is foolproof.)
'
DEF SEG=0
COM1ADD=PEEK(&H400)+256*PEEK(&H401) : COM2ADD=PEEK(&H402)+256*PEEK(&H403)
LPT1ADD=PEEK(&H408)+256*PEEK(&H409) : LPT2ADD=PEEK(&H40A)+256*PEEK(&H40B)
LPT3ADD=PEEK(&H40C)+256*PEEK(&H40D)
IF COM1ADD<>0 THEN PRINT "COM1 address = ";HEX$(COM1ADD);"h" : PRINT
IF COM2ADD<>0 THEN PRINT "COM2 address = ";HEX$(COM2ADD);"h" : PRINT
IF LPT1ADD<>0 THEN PRINT "LPT1 address = ";HEX$(LPT1ADD);"h" : PRINT
IF LPT2ADD<>0 THEN PRINT "LPT2 address = ";HEX$(LPT2ADD);"h"
PRINT
PRINT
PRINT "Press a key to continue...";
20 IF INKEY$="" THEN GOTO 20
PRINT
PRINT
PRINT
IF LPT3ADD<>0 THEN PRINT "LPT3 address = ";HEX$(LPT3ADD);"h" : PRINT
INREGS.AX=&H300
INREGS.DX=2
CALL INTERRUPTX(&H14,INREGS,OUTREGS)
COM3D=SGN(96-INT(OUTREGS.AX/256+.001))
INREGS.AX=&H300
INREGS.DX=3
CALL INTERRUPTX(&H14,INREGS,OUTREGS)
COM4D=SGN(96-INT(OUTREGS.AX/256+.001))
IF COM3D<0 THEN PRINT "COM3 detected" : PRINT
IF COM4D<0 THEN PRINT "COM4 detected" : PRINT
'
'  For non-PS/2 computers, look for rodent.  Detection of where mouse is
' attached is rigorous only if a "Mouse Systems" driver is loaded.  Even
' then, all that can be ascertained from querying the driver is whether
' it's COM1 or COM3, or COM2 or COM4.  If other tests indicate that one
' of the particular COM ports doesn't exist, it is then narrowed down
' further.
'
IF COMP$<>"PS/2" THEN
'
'  Interrupt 33 / subfunction AL = 0 (all calls to interrupt 33 use
' function AH = 0) returns information indicating whether a mouse driver
' is installed, and if so, how many buttons the mouse has.
'
INREGS.AX=0
CALL INTERRUPTX(&H33,INREGS,OUTREGS)
IF OUTREGS.AX=&HFFFF THEN
'
'  The mouse exists.  Get driver location and query it to see where mouse
' is.
'
NBUT$=LTRIM$(RTRIM$(STR$(OUTREGS.BX)))
'
'  Interrupt 21 / function 35 returns address of interrupt routines.
' (In this case, interrupt 33 is the mouse driver.)
'
INREGS.AX=&H3533
CALL INTERRUPTX(&H21,INREGS,OUTREGS)
'
'  Point basic to location of mouse driver.
'
DEF SEG=OUTREGS.ES
'
'  For Mouse Systems driver, COM interrupt is stored at an offset of 18h
' into the driver.  If the mouse is on COM1 or COM3, this interrupt will
' be C (hex).  For COM2/COM4, it will be Bh.
'
MINTRP=PEEK(OUTREGS.BX+18)
MCOM$="unidentifiable port"
IF MINTRP=11 THEN MCOM$="COM2 or COM4"
IF MINTRP=12 THEN MCOM$="COM1 or COM3"
IF MCOM$="unidentifiable port" THEN
'
'  Driver either isn't from Mouse Systems or else it's of some different
' version than assumed here.  Use other information to try and "second-
' guess" where the rodent is.
'
IF COM1ADD<>0 AND COM2ADD=0 THEN MCOM$="COM1 or COM3"
IF COM1ADD=0 AND COM2ADD<>0 THEN MCOM$="COM2 or COM4"
END IF
'
'  Try to use other information to narrow down mouse location.
'
IF MCOM$="COM1 or COM3" THEN IF COM3D>-1 THEN MCOM$="COM1"
IF MCOM$="COM1 or COM3" THEN IF COM1ADD=0 THEN MCOM$="COM3"
IF MCOM$="COM2 or COM4" THEN IF COM4D>-1 THEN MCOM$="COM2"
IF MCOM$="COM2 or COM4" THEN IF COM2ADD=0 THEN MCOM$="COM4"
PRINT NBUT$;"-button mouse installed via interrupt 33h on ";MCOM$
PRINT
END IF
END IF
'
'  Use interrupt 21 / function 52 to find out how many "block devices"
' are present.
'
INREGS.AX=&H5200
CALL INTERRUPTX(&H21,INREGS,OUTREGS)
DEF SEG=OUTREGS.ES
NBLOCK=PEEK(OUTREGS.BX+&H20)
PRINT LTRIM$(RTRIM$(STR$(NBLOCK)));" block devices (disk drives) detected"
PRINT
'
'  Find out if MSHERC is installed and also output video mode that existed
' before this program was called.
'
INREGS.AX=&HEF00
INREGS.DX=0
CALL INTERRUPTX(&H10,INREGS,OUTREGS)
IF OUTREGS.DX<>0 THEN PRINT "MSHERC detected" : PRINT
PRINT "Current bios video mode = ";HEX$(CMODE);"h"
ELSE
PRINT
PRINT "  You're using DOS 1.  Most of the functionallity of this program won't"
PRINT "work with your operating system."
END IF
PRINT
PRINT
'
'  If SCREEN 0 statement, above, changed video mode, put screen back into
' CMODE.
'
IF NEWMODE<>CMODE THEN
PRINT "Press a key to continue...";
30 IF INKEY$="" THEN GOTO 30
PRINT
INREGS.AX=CMODE
CALL INTERRUPTX(&H10,INREGS,OUTREGS)
'
'  Don't use QB END statement to terminate.  (It may change video mode.)
'  Use interrupt 21 / function 4C instead.
'
INREGS.AX=&H4C00
CALL INTERRUPTX(&H21,INREGS,OUTREGS)
END IF
END
'
'  This is a "functionized" version of code extracted from a more general
' numeric base conversion program by Robert B. Relf, (C) 1984.  This just
' uses the part of Mr. Relf's code that converts decimal to binary.
'
FUNCTION BIN$(NUM AS LONG)
DIM X AS INTEGER
IF NUM<0 THEN NUM=NUM+65536&
BIN1$=""
FOR X=15 TO 0 STEP -1
IF NUM>=(2^X) THEN
BIN1$=BIN1$+"1"
NUM=NUM-(2^X)
ELSE
BIN1$=BIN1$+"0"
END IF
NEXT X
BIN1$=LEFT$(BIN1$,8)+RIGHT$(BIN1$,8)
BIN$=BIN1$
END FUNCTION
'
'  This function inputs a string representing a binary number and converts
' it to a decimal number.
'
FUNCTION DEC%(B AS STRING)
DIM V AS LONG
N=LEN(B)
V=0
FOR I=1 TO N
V=V+2&^(I-1)*VAL(MID$(B,N-I+1,1))
NEXT I
DEC=CINT(V)
END FUNCTION
